import React from 'react';
import { CompositeLayer } from 'deck.gl/typed';
import { ArcLayer, LineLayer } from '@deck.gl/layers/typed';
import ArcLayerOptions from './ArcLayerOptions';
import * as d3 from 'd3';
import { getProperty } from '../../utlis';
import { BrushingExtension } from '@deck.gl/extensions/typed';
import { Edge, LayerProps } from '../../Types';

export const EdgeArcLayerConfig = {
  width: {
    fixed: true,
    accessor: 'getWidth',
    value: 1,
    condition: '',
  },
  color: {
    fixed: true,
    accessor: 'getColor',
    value: [255, 0, 0],
    condition: '',
    scaleRange: [
      [255, 0, 0],
      [0, 0, 255],
    ],
  },
  brushing: {
    active: false,
    radius: 300000,
  },
};

export class EdgeArcLayer extends CompositeLayer<LayerProps> {
  static type = 'Edge Arc';
  static layerOptions = EdgeArcLayerConfig;

  static generateLayerOptions(layer: any, updatedLayer: void, graphInfo: { [key: string]: any }, deleteLayer: void) {
    return <ArcLayerOptions layer={layer} updatedLayer={updatedLayer} graphInfo={graphInfo} deleteLayer={deleteLayer} />;
  }

  updateState({ props, oldProps, context, changeFlags }: { props: any; oldProps: any; context: any; changeFlags: any }) {
    const { graph, config } = props;

    if (!config.color.fixed) {
      const colorAttribute = graph.getGraphInfo().edgeAttributes[config.color.condition];
      const colorScale = this.createColorScale(config.color, colorAttribute);
      this.setState({ colorScale });
    }
  }

  createColorScale(colorProp: { [key: string]: any }, colorAttribute: { [key: string]: any }) {
    if (colorAttribute.dataType == 'boolean') {
      return d3.scaleOrdinal().domain(['0', '1']).range(colorProp.scaleRange);
    } else if (colorAttribute.dataType == 'number') {
      return d3.scaleLinear().domain(colorAttribute.values).range(colorProp.scaleRange);
    } else if (colorAttribute.dataType == 'string') {
      return d3
        .scaleOrdinal()
        .domain([...colorAttribute.values])
        .range(colorProp.scaleRange);
    } else {
      return null;
    }
  }

  getColor(edge: Edge, colorProps: { [key: string]: any }) {
    if (colorProps.fixed) {
      return colorProps.value !== undefined ? colorProps.value : colorProps.defaultValue;
    } else {
      const { condition } = colorProps;
      if (!condition) {
        return colorProps.defaultValue;
      }
      const value = getProperty(edge.attributes, condition);
      return this.state.colorScale(value);
    }
  }

  renderLayers() {
    const { graph, config, visible } = this.props;

    return new ArcLayer({
      hidden: visible,
      data: graph.getEdges(),
      pickable: true,
      getWidth: (d: any) => 1,
      extensions: [new BrushingExtension()],
      brushingEnabled: config.brushing.active,
      brushingRadius: 30000,
      getSourcePosition: (d: any) => graph.getNodeLocation(d.from),
      getTargetPosition: (d: any) => graph.getNodeLocation(d.to),
      getSourceColor: (d: any) => this.getColor(d, config.color),
      getTargetColor: (d: any) => this.getColor(d, config.color),
    });
  }
}

EdgeArcLayer.layerName = 'Edge Arc';
